import functools

import requests
from tenacity import retry, wait_exponential, stop_after_attempt

from peakventures.credentials import get_credentials_json, API_CREDENTIALS_NAME

BASE_URL_V2 = "https://live.adtechapi.xyz"


@functools.lru_cache(maxsize=1)
def _get_session() -> requests.Session:
    credentials = get_credentials_json(API_CREDENTIALS_NAME)

    session = requests.Session()

    auth_response = session.post("https://peakventures.us.auth0.com/oauth/token", credentials)
    auth_response.raise_for_status()

    auth_response_json = auth_response.json()

    session.headers = {
        "Authorization": f"{auth_response_json['token_type']} {auth_response_json['access_token']}"
    }

    return session


@retry(wait=wait_exponential(1), stop=stop_after_attempt(3))
def get_topic(topic: str) -> dict:
    """Get topic by id."""
    session = _get_session()

    response = session.get(f"{BASE_URL_V2}/topics/{topic}")
    response.raise_for_status()

    return response.json()


@retry(wait=wait_exponential(1), stop=stop_after_attempt(3))
def update_sellside_keywords_weights(topic: str, weights: dict) -> None:
    """Update sellside weights."""
    session = _get_session()

    response = session.put(f"{BASE_URL_V2}/topics/{topic}/keywords", json={
        "type": "sellside",
        "weights": weights
    })

    response.raise_for_status()


@retry(wait=wait_exponential(1), stop=stop_after_attempt(3))
def get_topic_afd_domains(topic: str) -> list:
    """Get topic AFD domains."""
    session = _get_session()

    response = session.get(f"{BASE_URL_V2}/topics/{topic}")
    response.raise_for_status()

    topic = response.json()

    return topic["properties"]["afdMappings"]


__all__ = [
    "get_topic",
    "get_topic_afd_domains",
    "update_sellside_keywords_weights"
]



import json
import os
from typing import Any

S3_CREDENTIALS_NAME = "db-s3"
API_CREDENTIALS_NAME = "db-api"


def _get_dbutils() -> Any:
    from pyspark.sql import SparkSession
    from pyspark.dbutils import DBUtils
    return DBUtils(SparkSession.getActiveSession())


def _get_credential(name: str) -> str:
    if os.getenv("DEVELOPMENT"):
        result = {
            S3_CREDENTIALS_NAME: os.getenv("S3_CREDENTIALS"),
            API_CREDENTIALS_NAME: os.getenv("API_CREDENTIALS")
        }.get(name)
    else:
        dbutils = _get_dbutils()
        result = dbutils.secrets.get("credentials", name)

    if not result:
        raise ValueError(f"Credentials {name} not found.")

    return result


def get_credentials(name: str) -> str:
    """Get credentials from the remote storage."""
    return _get_credential(name)


def get_credentials_json(name: str) -> Any:
    """Get credentials from the remote storage."""
    return json.loads(get_credentials(name))


__all__ = [
    "get_credentials",
    "get_credentials_json",
    "S3_CREDENTIALS_NAME",
    "API_CREDENTIALS_NAME"
]



import json
import logging

import requests

from websockets.sync import client as websocket

DEV_URL = "wss://dev.adtechapi.xyz/conversions/stream"
LIVE_URL = "wss://live.adtechapi.xyz/conversions/stream"

AUTH_URL = "https://peakventures.us.auth0.com/oauth/token"
AUTH_AUDIENCE = "https://api.peakventures.co"
AUTH_GRANT_TYPE = "client_credentials"

logger_handler = logging.StreamHandler()
logger_handler.setLevel(logging.INFO)
logger_handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logger_handler)


class ConversionsClient:
    client_id: str
    client_secret: str

    url: str

    initialized: bool = False

    connection: websocket.Connection
    conversions_number = 0

    def __init__(self: "ConversionsClient", client_id: str, client_secret: str, url: str = LIVE_URL) -> None:
        """
        Initialize Conversions Client.

        :param client_id: Auth0 client id.
        :param client_secret: Auth0 client secret.
        :param url: WebSocket API URL.
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.url = url

    def __enter__(self: "ConversionsClient") -> "ConversionsClient":
        """Enter context manager."""
        access_token = self.__get_access_token()

        self.connection = websocket.connect(self.url, additional_headers={
            "Authorization": f"Bearer {access_token}"
        })

        self.initialized = True

        logger.info("Connected to %s", self.url)

        return self

    def __exit__(self: "ConversionsClient", exc_type, exc_val, exc_tb) -> None:
        """Exit context manager."""
        logger.info("Streamed %d conversions", self.conversions_number)

        self.connection.close()

        logger.info("Disconnected from %s", self.url)

    def send(
            self: "ConversionsClient",
            buyside_network: str,
            buyside_network_account: str,
            pixel_id: str,
            pixel_event: str,
            click_id: str,
            conversion_date: int,
            user_agent: str,
            id_address: str,
            conversion_value: float) -> None:
        """Send conversion event."""

        if not self.initialized:
            raise RuntimeError("ConversionsClient is not initialized, use `with` statement")

        self.connection.send(json.dumps({
            "BuysideNetwork": buyside_network,
            "BuysideNetworkAccount": buyside_network_account,
            "PixelId": pixel_id,
            "PixelEvent": pixel_event,
            "ClickId": click_id,
            "ConversionDate": conversion_date,
            "UserAgent": user_agent,
            "IpAddress": id_address,
            "ConversionValue": conversion_value,
        }))

        self.conversions_number += 1

    def __get_access_token(self: "ConversionsClient") -> str:
        """Get JWT token from client credentials."""
        auth_response = requests.post(
            AUTH_URL,
            json={
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "audience": AUTH_AUDIENCE,
                "grant_type": AUTH_GRANT_TYPE
            },
        )

        auth_response.raise_for_status()

        auth_response_json = auth_response.json()

        logger.info("Got access token %s", auth_response_json["access_token"])

        return auth_response_json["access_token"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['peakventures']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.28.11,<2.0.0',
 'tenacity>=8.2.2,<9.0.0',
 'websockets>=11.0.3,<12.0.0']

setup_kwargs = {
    'name': 'peakventures',
    'version': '1.2.0',
    'description': 'PeakVentures Python Utilities for DataBricks',
    'long_description': 'None',
    'author': 'Volodymyr Smirnov',
    'author_email': 'volodymyr@peakventures.co',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json
from io import BytesIO
from typing import Any

import boto3

from peakventures.credentials import get_credentials_json, S3_CREDENTIALS_NAME

CONTAINER_NAME = "models"


def _store_model_s3(name: str, payload: str) -> None:
    credentials = get_credentials_json(S3_CREDENTIALS_NAME)

    s3 = boto3.client(
        "s3",
        endpoint_url=credentials["endpoint_url"],
        aws_access_key_id=credentials["access_key_id"],
        aws_secret_access_key=credentials["secret_access_key"],
    )

    with BytesIO(payload.encode("utf-8")) as buffer:
        s3.upload_fileobj(buffer, CONTAINER_NAME, name)


def store_model(name: str, payload: Any) -> None:
    """Store model on the remote storage."""
    json_payload = json.dumps(payload)

    _store_model_s3(name, json_payload)


__all__ = [
    "store_model"
]
